/*
 * Created on May 19, 2003
 *
 *  Copyright Intelligent ChoicePoint Inc. 2003
 */
 
package org.openl.util;

import java.io.File;
import java.util.Iterator;

import junit.framework.Assert;
import junit.framework.TestCase;

/**
 * @author snshor
 *
 */
public class TreeIteratorTest extends TestCase
{
	
	String[] x1 = {"aaa","bbb", "ccc"};
	String[] x2 = {"ddd","eee", "fff"};
	String[] x3 = {"ggg","hhh"};
	
	
	String[][] root = {x1, x2, x3};
	
	
	static class XTreeAdaptor implements TreeIterator.TreeAdaptor
	{
		public Iterator children(Object node)
		{
			if (node.getClass().isArray())
			{
				return OpenIterator.fromArray((Object[])node);
			}
			return null;
		}
	}
	
	
	TreeIterator create(int mode)
	{
		return new TreeIterator(this.root,new XTreeAdaptor(), mode);
	}
	

  /**
   * Constructor for TreeIteratorTest.
   * @param arg0
   */
  public TreeIteratorTest(String arg0)
  {
    super(arg0);
  }

  public void testHasNext()
  {
  	
  }

  public void testNext()
  {
  }

  public void testCount()
  {
		TreeIterator it = create(TreeIterator.DEFAULT);
		Assert.assertEquals(12, it.count());
  }

  /*
   * Test for int size()
   */
  public void testSize()
  {
		TreeIterator it = create(TreeIterator.DEFAULT);
		Assert.assertEquals(-1, it.size());
  	
  }
  
  public void testModes()
  {
		TreeIterator it = create(TreeIterator.BOTTOM_TOP);
		Assert.assertEquals(12, it.count());
		it = create(TreeIterator.BOTTOM_TOP);
		it.skip(2);
		Assert.assertEquals("ccc", it.next());


		it.skip(8);
		
		Assert.assertEquals(this.root, it.next());

		it = create(TreeIterator.RIGHT_TO_LEFT);
		it.skip(3);
		Assert.assertEquals("ggg", it.next());
		
		
		  	
  }
  
  public void testFile()
    throws Exception
  {
  	FileTreeIterator it = new FileTreeIterator(new File(".").getCanonicalFile(), 0);
  	
  	ISelector sel = new ASelector()
  	{
  		public boolean select(Object obj)
  		{
  			File f = (File) obj;
  			boolean res = f.getAbsolutePath().endsWith("TreeIteratorTest.java");
				Log.info(f.getAbsolutePath());
  			if (res)
  			  Log.info(f.getAbsolutePath());
  			return res;  
  		} 
  		
  	};
  	
  	Assert.assertEquals(it.select(sel).count(), 1);
  	
  }
  

  public void testSkip()
  {
  }

}
